<?php
namespace Eniture\StandardBoxSizes\Block\Adminhtml\Order\View\Tab;

class BoxSizesData extends \Magento\Backend\Block\Template implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var string
     */
    protected $_template = 'order/view/tab/boxsizesdata.phtml';

    /**
     * @var \Magento\Framework\Registry|null
     */
    private $coreRegistry = null;

    private $productloader;

    /**
     * BoxSizesData constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ProductFactory $productloader
     * @param \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ProductFactory $productloader,
        \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory,
        array $data = []
    ) {
        $this->coreRegistry     = $registry;
        $this->productloader   = $productloader;
        $this->_boxsizesFactory = $boxsizesFactory->create();
        
        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     */
    public function getOrder()
    {
        return $this->coreRegistry->registry('current_order');
    }

    /**
     * {@inheritdoc}
     */
    public function getTabLabel()
    {
        return __('Standard Box Sizes');
    }

    /**
     * {@inheritdoc}
     */
    public function getTabTitle()
    {
        return __('Standard Box Sizes');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @return string
     */
    public function getTabClass()
    {
        return 'ajax only';
    }

    /**
     * @return string
     */
    public function getClass()
    {
        return $this->getTabClass();
    }

    /**
     * @return string
     */
    public function getTabUrl()
    {
        return $this->getUrl('boxsizesdata/*/BoxSizesData', ['_current' => true]);
    }

    /**
     * @param $lastKey
     * @return string
     */
    function getServiceName($lastKey)
    {
        $code = '';
        switch ($lastKey) {
            case 'AIR':
                $code = 'fedexAirServices';
                break;
            case 'ORT':
                $code = 'fedexOneRate';
                break;
            case 'NML':
                $code = 'fedexServices';
                break;
            case 'FSP':
                $code = 'fedexSmartPost';
                break;
            case 'WWE':
                $code = 'wweServices';
                break;
            case 'UPS':
                $code = 'upsServices';
                break;
            default:
                break;
        }
        return $code;
    }

    /**
     * @param $details
     * @return int|void
     */
    public function binTotalItemsCount($details)
    {
        $items = 0;
        foreach ($details->bins_packed as $d) {
            $items += count($d->items);
        }
        return $items;
    }

    /**
     * @param $binData
     * @return bool
     */
    public function boxDims($binData)
    {
        return (isset($binData->w, $binData->h, $binData->d) && !($binData->w > 0 || $binData->h > 0 || $binData->d > 0)) ? true : false;
    }

    /**
     * @param $id
     * @return mixed
     */
    function boxAttr($id, $attr)
    {
        $boxsizeData = [];
        $boxCollection = $this->_boxsizesFactory->getCollection()->addFilter('box_id', ['eq' => $id]);
        
        foreach ($boxCollection as $box) {
            $boxsizeData[] = $box->getData();
        }

        return (isset($boxsizeData) && !empty($boxsizeData))?$boxsizeData[0][$attr]:'';
    }

    /**
     * @param $id
     * @return mixed
     */
    function boxWeight($id, $productWeight)
    {
        $boxWeight = $this->boxAttr($id, 'boxweight');
        $boxWeight = ($boxWeight === '') ? 0 : $boxWeight;
        return round($boxWeight + $productWeight, 3);
    }

    /**
     * @param $details
     * @return int|void
     */
    function getBinDataCount($details)
    {
//        $binDataCount = 0;
//        foreach ($details->bins_packed as $binDetails) {
//            $binDataCount += count($binDetails->bin_data);
//        }
//        return $binDataCount;
        return count((array)$details->bins_packed);
    }

    /**
     * @param $details
     * @param $zip
     */
    public function outputBinsPacked($details, $zip)
    {
        $mainBinImg = $boxOutput = $this->itemOwnPkg = '';
        $binCount = $packedItems = 1;
        $boxesOutputArr = [];
        $boxOutput .= "<div class='per-package'>";
        /* Bin packed details */
        $getBinDataCount = $this->getBinDataCount($details);
        
        foreach ($details->bins_packed as $binDetails) {
            $boxOutput = "";
            $dims = $this->boxDims($binDetails->bin_data);
            switch (true) {
                case (isset($binDetails->bin_data->type) && $binDetails->bin_data->type == "item" && $dims):
                    $index = "These items don't have dimensions and therefore couldn't be placed in a box. Shipping rates for these items were retrieved based on weight only.";
                    $packedItems = 3;
                    break;

                case (isset($binDetails->bin_data->type) && $binDetails->bin_data->type == "item"):
                    $index = "These items were quoted as shipping as their own package.";
                    $packedItems = 2;
                    break;
                default:
                    $index = "Packed items";
                    $packedItems = 1;
                    break;
            }(!isset($boxesOutputArr[$packedItems])) ? $boxOutput .= "<h3 class='packed-items'>$index</h3>" : "";

            $mainBinImg = $binDetails->image_complete;
            
            if ($packedItems == 2 || $packedItems == 3) {
                if ($packedItems == 2) {
                    $boxOutput .= '<div class="package-info">';
                    $boxOutput .= '<div class="package-dimensions">'
                    . '<p>' . floatval($binDetails->bin_data->d). ' X ' . floatval($binDetails->bin_data->w) . ' X ' . floatval($binDetails->bin_data->h) . '</p>'
                    . '</div>';
                    $boxOutput .= '<div class="package-complete-image">';
                    $boxOutput .= '<img class="package-complete-image-tag" src="' . $mainBinImg . '" />';
                    $boxOutput .= '</div>';
                    $boxOutput .= '</div>';
                } else {
                    $boxOutput .= '<div class="package-info">';
                    $boxOutput .= '<div class="package-complete-image">';
                    $boxOutput .= '<img class="package-complete-image-tag" src="' . $mainBinImg . '" />';
                    $boxOutput .= '</div>';
                    $boxOutput .='<div class="product-weight"> weight = '.$binDetails->bin_data->weight.' lbs</div> ';
                    $boxOutput .= '</div>';
                }
            } else {
                $boxOutput .= '<div class="en-package-' . $binCount . '">';
                $boxOutput .= '<div class="en-full-row">';
                $boxOutput .= '<div class="en-left before-steps-info">';
                $boxOutput .= '<p><b>Box ' . $binCount . ' of ' . $getBinDataCount . '</b></p>';
                $boxOutput .= '<p><b>Package Weight (lbs) : ' . $this->boxWeight($binDetails->bin_data->id, $binDetails->bin_data->weight) . '</b></p>';
                $boxOutput .= '<p><b>Number of items : ' . count($binDetails->items) . '</b></p>';
                $boxOutput .= '<p>' . $this->boxAttr($binDetails->bin_data->id, 'boxnickname') . '</p>';
                $boxOutput .= '<div class="package-dimensions">'
                . '<p>' . floatval($binDetails->bin_data->d). ' X ' . floatval($binDetails->bin_data->w) . ' X ' . floatval($binDetails->bin_data->h) . '</p>'
                . '</div>';
                $boxOutput .= '</div>';
                $boxOutput .= '<div class="package-complete-image">';
                $boxOutput .= '<img class="package-complete-image-tag" src="' . $mainBinImg . '" />';
                $boxOutput .= '</div>';
                $boxOutput .= '</div>';
                $boxOutput .= '</div>';
                $boxOutput .= $this->outputItemsPacked($binDetails, $zip);
            }
            
            $binCount++;
            (isset($boxesOutputArr[$packedItems])) ? $boxesOutputArr[$packedItems] .=  $boxOutput : $boxesOutputArr[$packedItems] = $boxOutput;
        }
        $boxOutput .= "</div>";
        echo implode("<br>", $boxesOutputArr);
    }

    /**
     * @param $binDetails
     * @param $zip
     * @return string
     */
    public function outputItemsPacked($binDetails, $zip)
    {
        $stepsOutput = "";
        $stepsOutput .= '<div class="package-steps-block">';
        $stepsOutput .= "<p><strong>Steps:</strong></p>";
        /* Items packed details */
        foreach ($binDetails->items as $itemDetails) {
            $product = $this->productloader->create()->load($itemDetails->id);
            $productTitle = $product->getName();
            $stepsOutput .= '<div class="package-steps-product">';
            $stepsOutput .= '<img class="en-prduct-steps-image" src="' . $itemDetails->image_sbs . '" />';
            $stepsOutput .= '<div class="en-product-steps-details">';
            $stepsOutput .= '<p class="en-prdouct-steps-name">' . $productTitle . '</p>';
            if (empty($this->itemOwnPkg)) {
                 $stepsOutput .= '<p class="en-prdouct-steps-dimensions">' . $itemDetails->d . ' X ' . $itemDetails->w . ' X ' . $itemDetails->h . '</p>';
            } else {
                $stepsOutput .= '<p class="en-prdouct-steps-dimensions">' . floatval($itemDetails->d) . ' X ' . floatval($itemDetails->w) . ' X ' . floatval($itemDetails->h) . '</p>';
            }
             $stepsOutput .= '</div>';
            $stepsOutput .= '</div>';
        }
        /* Clear the float effect */
        $stepsOutput .= '<div class="en-clear"></div>';
        $stepsOutput .= '</div>';
        
        return $stepsOutput;
    }
}
