<?php

namespace Eniture\StandardBoxSizes\Controller\Box;

use \Magento\Framework\App\Action\Action;

class SaveBoxsize extends Action
{
    private $connection;
    private $boxSizesTable;
    private $boxsizesFactory;

    /**
     * SaveBoxsize constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\ResourceConnection $resource,
        \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory
    ) {
        $this->connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->boxSizesTable = $resource->getTableName('boxsizes');
        $this->boxsizesFactory = $boxsizesFactory->create();
        parent::__construct($context);
    }

    /**
     * Save Box funtion
     */
    function execute()
    {
        $updateQry = $insertQry = 0;
        $postdata = $this->getRequest()->getPostValue();
        foreach ($postdata as $key => $post) {
            $saveBoxData[$key] = $post;
        }
        $rawBoxId = filter_var($postdata['box-id'], FILTER_SANITIZE_NUMBER_INT);
        $boxId    = isset($rawBoxId) ? intval($rawBoxId) : "";
        $boxData = $this->boxInsertArray($saveBoxData);
        $validateData   = $this->validatBoxPostData($boxData);
        extract($validateData);
        $edited = $isAvailable = false;
        $isDimExist = $this->checkBoxsizeList($validateData);
        $isNickname = $this->checkNickname($boxnickname);

        if (empty($isNickname)) {
            $isAvailable = true;
        }elseif ($boxId == $isNickname[0]['box_id'] && empty($isDimExist)){
            $isAvailable = true;
        }elseif (isset($boxData['oneRateId']) && $boxData['oneRateId'] != $isNickname[0]['oneRateId']) {
            $isAvailable = true;
        }

        if (!(isset($validateData['error']))) {

            if ($boxId && $isAvailable) {
                $updateQry = $this->connection->update("$this->boxSizesTable", $validateData, "box_id='".$boxId."'");
                $edited = true;
            } else {
                if ($isAvailable) {
                    $insertQry = $this->connection->insert("$this->boxSizesTable", $validateData);
                    $edited = true;
                }
            }
        }
        if(!$edited){
            $edited = $boxId != $isNickname[0]['box_id'] ? 'error' : $edited;
        }
        if ($insertQry == 0) {
            $lastid = $boxId;
        } else {
            $lastid = $this->connection->lastInsertId();
            $updateQry = $lastid;
        }

        $available = ($boxavailable == 1)?'Yes':'No';
        $boxList = ['nickname' => $boxnickname, 'length' => $boxlength, 'width' => $boxwidth, 'height' => $boxheight,
            'maxweight' => $boxmaxweight,'boxweight' => $boxweight, 'boxavailable' => $available,'id' => $lastid,
            'insertid' => $insertQry,
            'updateid' => $updateQry];
        if ($boxId || $edited) {
            $boxList['edited'] = $edited;
        }
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(json_encode($boxList));
    }

    /**
     * Data Array
     * @param $inputData
     * @return array
     */

    function boxInsertArray($inputData)
    {
        $dataArr = [
            'box_id'         => $inputData['box-id'],
            'boxnickname'    => $inputData['boxnickname'],
            'boxlength'      => $inputData['boxlength'],
            'boxwidth'       => $inputData['boxwidth'],
            'boxheight'      => $inputData['boxheight'],
            'boxmaxweight'   => $inputData['boxmaxweight'],
            'boxweight'      => $inputData['boxweight'],
            'boxavailable'   => $inputData['boxavailable'] == 'on' ? 1:0,
            'boxfee'         => $inputData['boxfee'],
        ];
        if($inputData['oneRateId'] != ''){
            $dataArr['fedex_box_type']  = $inputData['fedexBoxType'];
            $dataArr['oneRateId']       = $inputData['oneRateId'];
            $dataArr['type']            = 'onerate';
        }
        return $dataArr;
    }

    /**
     * validate Input Post
     * @param $sPostData
     * @return mixed
     */
    function validatBoxPostData($sPostData)
    {
        foreach ($sPostData as $key => $tag) {
            $check_characters = (!$key == 'boxnickname') ? preg_match('/[#$%@^&_*!()+=\-\[\]\';,.\/{}|":<>?~\\\\]/', $tag) : '';
            if ($check_characters != 1) {
                if ($key === 'boxnickname') {
                    $data[$key] = $tag;
                } else {
                    $data[$key] = preg_replace('/\s+/', '', $tag);
                }
            } else {
                $data['error'] = 'Error';
            }
        }
        return $data;
    }

    public function checkBoxsizeList($validatedData)
    {
        extract($validatedData);
        $boxData = [];
        $boxCollection = $this->boxsizesFactory->getCollection()
            ->addFilter('boxlength', ['eq' => $boxlength])
            ->addFilter('boxwidth', ['eq' => $boxwidth])
            ->addFilter('boxheight', ['eq' => $boxheight])
            ->addFilter('boxmaxweight', ['eq' => $boxmaxweight])
            ->addFilter('boxweight', ['eq' => $boxweight])
            ->addFilter('boxavailable', ['eq' => $boxavailable])
            ->addFilter('boxfee', ['eq' => $boxfee]);
        foreach($boxCollection as $boxes){
            $boxData[] = $boxes->getData();
        }

        return $boxData;
    }

    public function checkNickname($boxnickname)
    {
        $getNickname = [];
        $boxCollection = $this->boxsizesFactory->getCollection()
            ->addFilter('boxnickname', ['eq' => $boxnickname]);

        foreach($boxCollection as $boxes){
            $getNickname[] = $boxes->getData();
        }

        return $getNickname;
    }
}
