<?php
/**
 * Box Sizes
 * @package     Box Sizes
 * @author      Eniture-Technology
 */
/**
 * class that instantiate guide section html block template
 */
namespace Eniture\StandardBoxSizes\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class Data extends AbstractHelper
{
    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    private $connection;
    /**
     * @var string
     */

    
    private $planStatusReqUrl = 'https://eniture.com/ws/addon/bin-packaging/index.php';

    /**
     * @var \Magento\Framework\HTTP\Client\CurlgetEnabledOneRateServices
     */
    private $curl;

    public $cacheManager;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\App\Config\ConfigResource\ConfigInterface $resourceConfig
     * @param \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\App\Cache\Manager $cacheManager
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\App\Config\ConfigResource\ConfigInterface $resourceConfig,
        \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Cache\Manager $cacheManager
    ) {
        $this->connection          =  $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->coreConfigTable     = $resource->getTableName('core_config_data');
        $this->_boxSizesTable     = $resource->getTableName('boxsizes');
        $this->request = $context->getRequest();
        $this->scopeConfig = $context->getScopeConfig();
        $this->resourceConfig = $resourceConfig;
        $this->boxsizesFactory = $boxsizesFactory->create();
        $this->curl = $curl;
        $this->cacheManager = $cacheManager;
        parent::__construct($context);
    }
    
    public function getBoxFactory()
    {
        return $this->boxsizesFactory;
    }

    /**
     * @param $licenseKey
     * @return array
     */
    public function boxSizesDataHandling($licenseKey)
    {
        $this->enitureModuleLicenseKey  = $licenseKey;
        $data = $this->getCurrentPlanStatus();
        return $data;
    }
    
    /**
     * getCurrentPlanStatus
     * @return array
     */
    public function getCurrentPlanStatus()
    {
        $trialStart = $this->scopeConfig->getValue('binBox/trial/use', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if ($trialStart == '0') {
            $trialActive = $this->getRequestArray('c', 'TR');
            if (isset($trialActive->severity) && $trialActive->severity == 'SUCCESS') {
                $this->resourceConfig->saveConfig(
                    'binBox/trial/use',
                    '1',
                    \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
                    \Magento\Store\Model\Store::DEFAULT_STORE_ID
                );
            }
        }
        
        $planStatus = $this->getRequestArray('s', '');
        return $this->setPlanStatusData($planStatus);
    }

    /**
     * @return string
     */
    public function checkSmallModuleTrial()
    {
        $smallTrialMsg = '';
        $smallModuleStatus = $this->getRequestArray('pluginType', '');
        if (isset($smallModuleStatus->severity) && $smallModuleStatus->severity == 'SUCCESS' && $smallModuleStatus->pluginType == 'trial') {
            $smallTrialMsg = 'The Small Package Quotes module must have active paid license to continue to use this feature.';
        }
        return $smallTrialMsg;
    }

    /**
     * @param $statusResponse
     * @return array
     */
    public function setPlanStatusData($statusResponse)
    {
        if (isset($statusResponse->severity) && $statusResponse->severity == 'SUCCESS') {
            $this->showStatus = 'yes';
            $this->statusMsg = 'SUCCESS';
            $planStatus = isset($statusResponse->status)?$statusResponse->status:'';
            
            if ($planStatus) {
                $this->packageHits          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageHits))) ? $planStatus->subscribedPackage->packageHits : "NA";
                $this->packageDuration      = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageDuration))) ? $planStatus->subscribedPackage->packageDuration : "NA";
                $this->packageCost          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageCost))) ? "($" .$planStatus->subscribedPackage->packageCost. ")": "NA";
                $this->subscriptionTime     = (isset($planStatus->subscriptionInfo) && (!empty($planStatus->subscriptionInfo->subscriptionTime))) ? "Start date: " . $this->formateDateTime($planStatus->subscriptionInfo->subscriptionTime) : "NA";
                $this->expiryTime           = (isset($planStatus->subscriptionInfo) && (!empty($planStatus->subscriptionInfo->expiryTime))) ? "End date: " . $this->formateDateTime($planStatus->subscriptionInfo->expiryTime) : "NA";

                $this->subscriptionStatus   = (isset($planStatus->subscriptionInfo) && $planStatus->subscriptionInfo->subscriptionStatus == '1')?'active':'deactive';
                
                $this->consumedHits         = (isset($planStatus->subscribedPackageHitsStatus) && (!empty($planStatus->subscribedPackageHitsStatus->consumedHits))) ? $planStatus->subscribedPackageHitsStatus->consumedHits : "0";
                $this->consumedHitsPrcent   = (isset($planStatus->subscribedPackageHitsStatus) && (!empty($planStatus->subscribedPackageHitsStatus->consumedHitsPrcent))) ? $planStatus->subscribedPackageHitsStatus->consumedHitsPrcent. "%" : "NA";
                $this->lastUsageTime        = "(". $planStatus->lastUsageTime .")";
            
                $this->packageSCAC          = (isset($planStatus->subscribedPackage) && (!empty($planStatus->subscribedPackage->packageSCAC))) ? $planStatus->subscribedPackage->packageSCAC: "NA";
                $this->checkTrail           = (isset($planStatus->subscribedPackage)? $planStatus->subscribedPackage->subscribedPackageName : "NA" );
                
                $this->nextSubcribedPackage       = (isset($planStatus->nextSubcribedPackage) && (!empty($planStatus->nextSubcribedPackage->nextSubscriptionSCAC))) ? $planStatus->nextSubcribedPackage->nextSubscriptionSCAC: "NA";
                $this->nextSubcribedPackagestatus = (isset($planStatus->nextSubcribedPackage) && (!empty($planStatus->nextSubcribedPackage->nextToBeChargedStatus))) ? $planStatus->nextSubcribedPackage->nextToBeChargedStatus: "NA";
            }
        } else {
            $this->showStatus = 'no';
            $this->statusMsg = isset($statusResponse->Message) ? $statusResponse->Message : 'Your current subscription is expired.';
        }
            
        $this->ListOfPackages       = isset($statusResponse->ListOfPackages) ? $statusResponse->ListOfPackages->Info : '';

        return $this->setPlanDataArr();
    }

    /**
     * @return array
     */
    function setPlanDataArr()
    {
        $arr = [
            'showStatus'        => $this->showStatus,
            'statusMsg'         => $this->statusMsg,
            'packageHits'       => (isset($this->packageHits))?$this->packageHits:'NA',
            'packageDuration'   => (isset($this->packageDuration))?$this->packageDuration:'NA',
            'packageCost'       => (isset($this->packageCost))?$this->packageCost:'NA',
            'subscriptionTime'  => (isset($this->subscriptionTime))?$this->subscriptionTime:'NA',
            'expiryTime'        => (isset($this->expiryTime))?$this->expiryTime:'NA',
            'consumedHits'      => (isset($this->consumedHits))?$this->consumedHits:'NA',
            'consumedHitsPrcent' => (isset($this->consumedHitsPrcent))?$this->consumedHitsPrcent:'NA',
            'lastUsageTime'     => (isset($this->lastUsageTime))?$this->lastUsageTime:'NA',
            'packageSCAC'       => (isset($this->packageSCAC))?$this->packageSCAC:'NA',
            'ListOfPackages'    => $this->ListOfPackages,
            'checkTrail'        => (isset($this->checkTrail)? $this->checkTrail : "NA" ),
            'nextSubcribedPackage'        => (isset($this->nextSubcribedPackage)? $this->nextSubcribedPackage : "NA" ),
            'nextSubcribedPackagestatus'  => (isset($this->nextSubcribedPackagestatus)? $this->nextSubcribedPackagestatus : "NA" ),
            'subscriptionStatus'  => (isset($this->subscriptionStatus)? $this->subscriptionStatus : "NA" ),
            
        ];

        return $arr;
    }

    /**
     * @param $datetime
     * @return false|string
     */
    public function formateDateTime($datetime)
    {
        $date = date_create($datetime);
        return date_format($date, "M. d, Y");
    }

    /**
     * @return string
     */
    public function boxUseSuspended()
    {
        $boxSuspend = $this->scopeConfig->getValue('binPackaging/suspend/value', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        return $boxSuspend;
    }

    /**
     * @return array
     */
    public function getBoxSizes()
    {
        $getBoxes = [];
        $boxCollection = $this->boxsizesFactory->getCollection();
        foreach ($boxCollection as $value) {
            $data = $value->getData();
            $getBoxes[] = $data;
        }
        return $getBoxes;
    }

    /**
     * @param $action
     * @param $package
     * @return array
     */
    protected function getRequestArray($action, $package)
    {
        $postArr = [
            'platform'      => 'magento2',
            'request_key'   => md5(microtime() . rand()),
            'action'        => $action,
            'package'       => $package,
            'domain_name'   => $this->request->getServer('SERVER_NAME'),
            'license_key'   => $this->enitureModuleLicenseKey,
        ];
        return $this->sendRequest($postArr);
    }

    /**
     * @return array
     */
    public function fillBoxingData()
    {
        $boxingData = [];
        $boxCollection = $this->boxsizesFactory->getCollection();
        foreach ($boxCollection as $value) {
            $boxes = $value->getData();
            if ($boxes['boxavailable'] == 1) {
                if (isset($boxes['fedex_box_type']) && !empty($boxes['fedex_box_type'])) {
                    $mainIndex = 'fedex_bins';
                } else {
                    $mainIndex = 'bins';
                }

                $boxingData[$mainIndex][$boxes['box_id']] = $this->createBinsArr($boxes);
            }
        }
        
        return $boxingData;
    }
    
    /**
     *
     * @param type $boxes
     * @return type
     */
    function createBinsArr($boxes)
    {
        return [
                'nickname'          => $boxes['boxnickname'],
                'fedex_box_type'    => $boxes['fedex_box_type'],
                'w'                 => floatval($boxes['boxwidth']),
                'h'                 => floatval($boxes['boxheight']),
                'd'                 => floatval($boxes['boxlength']),
                'id'                => floatval($boxes['box_id']),
                'max_wg'            => floatval($boxes['boxmaxweight']),
                'box_weight'        => floatval($boxes['boxweight']),
        ];
    }
    /**
     *
     * @return type
     */
    public function oneRateData()
    {
        return [
            [9.5 , 12.5 , 0.5 , 10 , 0 , 'FEDEX_ENVELOPE' , 0 , 'FedEx Envelope'],
            [9.5 , 15.5 , 0.5 , 10 , 0 , 'FEDEX_ENVELOPE' , 1 , 'FedEx Reusable Envelope'],
            [10.25 , 12.75 , 1.5 , 50 , 0 , 'FEDEX_PAK' , 2 , 'FedEx Pak - Small'],
            [12 , 15.5 , 1.5 , 50 , 0 , 'FEDEX_PAK' , 3 , 'FedEx Pak - Large'],
            [11.75 , 14.75 , 1.25 , 50 , 0 , 'FEDEX_PAK' , 4 , 'FedEx Pak - Padded'],
            [10 , 14.5 , 1.25 , 50 , 0 , 'FEDEX_PAK' , 5 , 'FedEx Pak - Reusable'],
            [10.875 , 1.5 , 12.375 , 50 , 0 , 'FEDEX_SMALL_BOX' , 6 , 'FedEx Small Box'],
            [8.75 , 2.625 , 11.25 , 50 , 0 , 'FEDEX_SMALL_BOX' , 7 , 'FedEx Small Box'],
            [11.5 , 2.375 , 13.25 , 50 , 0 , 'FEDEX_MEDIUM_BOX' , 8 , 'FedEx Medium Box'],
            [8.75 , 4.375 , 11.25 , 50 , 0 , 'FEDEX_MEDIUM_BOX' , 9 , 'FedEx Medium Box'],
            [12.375 , 3 , 17.5 , 50 , 0 , 'FEDEX_LARGE_BOX' , 10 , 'FedEx Large Box'],
            [8.75 , 7.75 , 11.25 , 50 , 0 , 'FEDEX_LARGE_BOX' , 11 , 'FedEx Large Box'],
            [11.875 , 10.75 , 11 , 50 , 0 , 'FEDEX_EXTRA_LARGE_BOX' , 12 , 'FedEx Extra Large Box'],
            [15.75 , 14.125 ,6 , 50 , 0 , 'FEDEX_EXTRA_LARGE_BOX' , 13 , 'FedEx Extra Large Box'],
        ];
    }

    /**
     * @return array
     */
    public function getEnabledOneRateServices()
    {
        $boxes = $this->getBoxSizes();
        $enabledOneRate = [];
        foreach ($boxes as $box) {
            if($box['type'] == 'onerate' && $box['boxavailable'] == 1) {
                $enabledOneRate[] = $box;
            }
        }
        return $enabledOneRate;
    }
    
    /**
     * This function send request and return responce
     * $isAssocArray Paramiter When TRUE, then returned objects will
     * be converted into associative arrays, otherwise its an object
     * @param $url
     * @param $postData
     * @param $isAssocArray
     * @return array
     */
    public function sendRequest($postData, $isAssocArray = false)
    {
        $fieldString = json_encode($postData);
        $this->curl->post($this->planStatusReqUrl, $fieldString);
        $output = $this->curl->getBody();
        $result = json_decode($output, $isAssocArray);
        return $result;
    }


    public function clearCache()
    {
        $this->cacheManager->flush($this->cacheManager->getAvailableTypes());

        // or this
        $this->cacheManager->clean($this->cacheManager->getAvailableTypes());
    }
}
